--[[
Name: RollCall-1.0
Revision: $Rev: 25796 $
Author: Cameron Kenneth Knight (ckknight@gmail.com)
Website: http://wiki.wowace.com/index.php/RollCall-1.0
Documentation: http://wiki.wowace.com/index.php/RollCall-1.0
SVN: svn://svn.wowace.com/wowace/trunk/RollCall-1.0/RollCall-1.0
Description: A library to provide an easy way to get information about guild members.
Dependencies: AceLibrary, AceOO-2.0, AceEvent-2.0, Babble-Zone-2.2 (optional),
				Babble-Class-2.2 (optional)
License: LGPL v2.1
]]

local MAJOR_VERSION = "RollCall-1.0"
local MINOR_VERSION = "$Revision: 25796 $"

-- This ensures the code is only executed if the libary doesn't already exist, or is a newer version
if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0.") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0.") end

local AceEvent = AceLibrary:GetInstance("AceEvent-2.0")
local BZ, BC

local RollCall = {}

local IsInGuild = IsInGuild
local GuildRoster = GuildRoster
local select = select
local next = next

local new, del
do
	local cache = {}
	function new(...)
		local t = next(cache)
		if t then
			cache[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			return t
		else
			return { ... }
		end
	end
	
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		cache[t] = true
		return nil
	end
end

local RANK_ID = 1
local RANK_INDEX_ID = 2
local LEVEL_ID = 3
local CLASS_ID = 4
local ZONE_ID = 5
local STATUS_ID = 6
local NOTE_ID = 7
local OFFICER_NOTE_ID = 8

local players = {}
local playersOnline = 0
local playersTotal = 0

function RollCall:PLAYER_GUILD_UPDATE(arg1)
	if arg1 and arg1 ~= "player" then return end
	
	if IsInGuild() then
		if not self:IsBucketEventRegistered("GUILD_ROSTER_UPDATE") then
			self:ScheduleEvent("RollCall10-GuildRoster", GuildRoster, 15)
			self:RegisterBucketEvent("GUILD_ROSTER_UPDATE", 1)
		end
		GuildRoster()
		self:TriggerEvent("RollCall10_Joined")
	else
		if self:IsBucketEventRegistered("GUILD_ROSTER_UPDATE") then
			self:CancelScheduledEvent("RollCall10-GuildRoster")
			self:UnregisterBucketEvent("GUILD_ROSTER_UPDATE")
		end
		self:TriggerEvent("RollCall10_Left")
	end
end

local tmp = {}
function RollCall:GUILD_ROSTER_UPDATE()
	if IsInGuild() then
		local players = players
		for k, v in pairs(players) do
			players[k] = del(v)
			tmp[k] = true
		end
		playersOnline = 0
		playersTotal = GetNumGuildMembers(true)
		
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status
		for i = 1, playersTotal do
			name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i)
			if note == "" then
				note = nil
			end
			if officernote == "" then
				officernote = nil
			end
			if online then
				playersOnline = playersOnline + 1
				if name then
					players[name] = new(rank or UNKNOWN, rankIndex or -1, level or -1, class or UNKNOWN, zone or UNKNOWN, status, note, officernote)
					if tmp[name] then
						tmp[name] = nil
					else
						self:TriggerEvent("Roster10_MemberConnected", name)
					end
				end
			end
		end
		for k in pairs(tmp) do
			tmp[k] = nil
			self:TriggerEvent("Roster10_MemberDisconnected", k)
		end
		
		if self:IsBucketEventRegistered("GUILD_ROSTER_UPDATE") then
			self:ScheduleEvent("RollCall10-GuildRoster", GuildRoster, 15)
		end
	else
		for k, v in pairs(players) do
			players[k] = del(v)
		end
		playersOnline = 0
		playersTotal = 0
	end
	self:TriggerEvent("RollCall10_Updated")
end

function RollCall:GetNumOnline()
	return playersOnline
end

function RollCall:GetNumTotal()
	return playersTotal
end

local playerName = UnitName("player")

function RollCall:HasMember(name)
	local p = players[name or playerName]
	return p and true or false
end

function RollCall:GetRank(name)
	local p = players[name or playerName]
	return p and p[RANK_ID]
end

function RollCall:GetRankIndex(name)
	local p = players[name or playerName]
	return p and p[RANK_INDEX_ID]
end

function RollCall:GetLevel(name)
	local p = players[name or playerName]
	return p and p[LEVEL_ID]
end

function RollCall:GetClass(name)
	local p = players[name or playerName]
	return p and p[CLASS_ID]
end

function RollCall:GetClassColor(name)
	if not BC then
		self:error("Cannot call `GetClassColor' without Babble-Class-2.2 loaded.")
	end
	local p = players[name or playerName]
	local class = p and p[CLASS_ID]
	if class then
		return BC:GetColor(class)
	else
		return 1,1,1--0.8, 0.8, 0.8
	end
end

function RollCall:GetClassHexColor(name)
	if not BC then
		self:error("Cannot call `GetClassHexColor' without Babble-Class-2.2 loaded.")
	end
	local p = players[name or playerName]
	local class = p and p[CLASS_ID]
	if class then
		return BC:GetHexColor(class)
	else
		return "ffffff"--"cccccc"
	end
end

function RollCall:GetZone(name)
	local p = players[name or playerName]
	return p and p[ZONE_ID]
end

function RollCall:GetEnglishZone(name)
	if not BZ then
		self:error("Cannot call `GetEnglishZone' without Babble-Zone-2.2 loaded.")
	end
	local p = players[name or playerName]
	local zone = p and p[ZONE_ID]
	if zone ~= UNKNOWN then
		return BZ:GetReverseTranslation(zone)
	else
		return "Unknown"
	end
end

function RollCall:GetStatus(name)
	local p = players[name or playerName]
	return p and p[STATUS_ID]
end

function RollCall:GetNote(name)
	local p = players[name or playerName]
	return p and p[NOTE_ID]
end

function RollCall:GetOfficerNote(name)
	local p = players[name or playerName]
	return p and p[OFFICER_NOTE_ID]
end

function RollCall:GetGuildName()
	return (GetGuildInfo('player'))
end

local sorts; sorts = {
	NAME = function(a, b)
		return a < b
	end,
	CLASS =	function(a, b)
		local a_p, b_p = players[a], players[b]
		if a_p[CLASS_ID] < b_p[CLASS_ID] then
			return true
		elseif a_p[CLASS_ID] > b_p[CLASS_ID] then
			return false
		else
			if a_p[LEVEL_ID] < b_p[LEVEL_ID] then
				return true
			elseif a_p[LEVEL_ID] > b_p[LEVEL_ID] then
				return false
			else
				return a < b
			end
		end
	end,
	LEVEL =	function(a,b)
		local a_p, b_p = players[a], players[b]
		if a_p[LEVEL_ID] < b_p[LEVEL_ID] then
			return true
		elseif a_p[LEVEL_ID] > b_p[LEVEL_ID] then
			return false
		else
			if a_p[CLASS_ID] < b_p[CLASS_ID] then
				return true
			elseif a_p[CLASS_ID] > b_p[CLASS_ID] then
				return false
			else
				return a < b
			end
		end
	end,
	ZONE = function(a, b)
		local a_p, b_p = players[a], players[b]
		if a_p[ZONE_ID] < b_p[ZONE_ID] then
			return true
		elseif a_p[ZONE_ID] > b_p[ZONE_ID] then
			return false
		else
			return sorts.CLASS(a, b)
		end
	end,
	RANK = function(a, b)
		if a_p[RANK_ID] < b_p[RANK_ID] then
			return true
		elseif a_p[RANK_ID] > b_p[RANK_ID] then
			return false
		else
			return sorts.CLASS(a, b)
		end
	end,
}

local iter = function(t)
	local n = t.n
	n = n + 1
	t.n = n
	return t[n] or del(t)
end
function RollCall:GetIterator(sort)
	self:argCheck(sort, 2, "string", "nil")
	local sortFunc = sorts[sort or "NAME"]
	if not sortFunc then
		self:error('Argument #2 must be "NAME", "LEVEL", "CLASS", "ZONE", "RANK", or nil, got %q.', sort)
	end
	
	local tmp = new()
	for k in pairs(tmp) do
		tmp[k] = nil
	end
	
	for k in pairs(players) do
		tmp[#tmp+1] = k
	end
	
	table.sort(tmp, sortFunc)
	tmp.n = 0
	
	return iter, tmp, nil
end

local function external(self, major, instance)
	if major == "Babble-Class-2.2" then
		BC = instance
	elseif major == "Babble-Zone-2.2" then
		BZ = instance
	end
end

local function activate(self, oldLib, oldDeactivate)
	RollCall = self
	
	AceEvent:embed(self)
	
	self:UnregisterAllEvents()
	
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	
	if IsInGuild() then
		self:ScheduleEvent("RollCall10-GuildRoster", GuildRoster, 15)
		self:RegisterBucketEvent("GUILD_ROSTER_UPDATE", 1)
		GuildRoster()
	end
	
	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end

AceLibrary:Register(RollCall, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)
